#ifndef _IFLINK_H
#define _IFLINK_H

#include <stdio.h>
#ifdef __MINT__

#define SIOCSIFLINK	(('S' << 8) | 11)

struct iflink {
	char	ifname[16];
	char	device[128];
};

#define SIOCSLNKFLAGS	(('S' << 8) | 25)
#define SIOCGLNKFLAGS	(('S' << 8) | 26)

#define SLF_COMPRESS	0x08
#define SLF_AUTOCOMP	0x10

extern int	if_link		(char *, char *);
extern int	if_getlnkflags	(char *, short *);
extern int	if_setlnkflags	(char *, short);

#endif
#endif
